"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Generated by CoffeeScript 2.7.0
(function () {
  var AWS, MonitoringCenterObserver, Promise, _, debug, retry;

  _ = require("lodash");

  Promise = require("bluebird");

  retry = require("bluebird-retry");

  debug = require("debug")("notification-processor:observers:monitor-center");

  AWS = require("aws-sdk");

  module.exports = MonitoringCenterObserver = function () {
    function MonitoringCenterObserver(_ref) {
      var sender = _ref.sender,
          clientId = _ref.clientId,
          app = _ref.app,
          job = _ref.job,
          _ref$propertiesToOmit = _ref.propertiesToOmit,
          propertiesToOmit = _ref$propertiesToOmit === undefined ? "auth" : _ref$propertiesToOmit,
          _ref$connection = _ref.connection,
          accessKeyId = _ref$connection.accessKeyId,
          secretAccessKey = _ref$connection.secretAccessKey,
          monitoringCenterBucket = _ref$connection.monitoringCenterBucket,
          region = _ref$connection.region;

      _classCallCheck(this, MonitoringCenterObserver);

      this.uploadTrackingFile = this.uploadTrackingFile.bind(this);
      this.sender = sender;
      this.clientId = clientId;
      this.app = app;
      this.job = job;
      this.propertiesToOmit = propertiesToOmit;
      this.s3 = new AWS.S3({ accessKeyId: accessKeyId, secretAccessKey: secretAccessKey, region: region });
      this.bucket = monitoringCenterBucket;
      this.uploadToS3 = Promise.promisify(this.s3.upload).bind(this.s3);
    }

    _createClass(MonitoringCenterObserver, [{
      key: "listenTo",
      value: function listenTo(observable) {
        var _this = this;

        observable.on("unsuccessful_non_retryable", function (payload) {
          return _this.uploadTrackingFile(payload, "unsuccessful_non_retryable");
        });
        observable.on("unsuccessful", function (payload) {
          return _this.uploadTrackingFile(payload, "unsuccessful");
        });
        observable.on("started", function (payload) {
          return _this.uploadTrackingFile(payload, "started");
        });
        return observable.on("successful", function (payload) {
          return _this.uploadTrackingFile(payload, "successful");
        });
      }
    }, {
      key: "uploadTrackingFile",
      value: function uploadTrackingFile(_ref2, eventType) {
        var _this2 = this;

        var id = _ref2.id,
            notification = _ref2.notification,
            error = _ref2.error;

        return this._mapper(id, notification, error, eventType).then(function (_ref3) {
          var key = _ref3.key,
              body = _ref3.body;

          var __uploadToS3, uploadParams;
          if (!key || !body) {
            return;
          }
          uploadParams = {
            Bucket: _this2.bucket,
            Key: key,
            Body: body
          };
          debug("Uploading file " + uploadParams.Key + " to bucket " + uploadParams.Bucket);
          __uploadToS3 = function __uploadToS3() {
            return _this2.uploadToS3(uploadParams);
          };
          return retry(__uploadToS3, {
            throw_original: true
          }).tap(function () {
            return debug("Uploaded file " + uploadParams.Key + " to bucket " + uploadParams.Bucket);
          }).catch(function (e) {
            // We'll do nothing with this error
            return debug("Error uploading file " + uploadParams.Key + " to bucket " + uploadParams.Bucket + " %o", e);
          });
        });
      }
    }, {
      key: "_mapper",
      value: function _mapper(id, notification, err, eventType) {
        var _this3 = this;

        var ref, theRequest;
        if (!(notification != null ? (ref = notification.message) != null ? ref.EventId : void 0 : void 0)) {
          return Promise.resolve({});
        }
        theRequest = _.get(err, "detail.request") || _.get(err, "cause.detail.request");
        return Promise.props({
          resource: Promise.method(this.sender.resource)(notification),
          user: Promise.method(this.sender.user)(notification)
        }).then(function (_ref4) {
          var resource = _ref4.resource,
              user = _ref4.user;

          var ref1, ref2;
          return {
            key: user + "/" + notification.message.EventId + "/" + id + "/" + _this3.app + "|" + _this3.job + "|" + eventType + "|" + new Date().toISOString(),
            body: JSON.stringify({
              eventId: notification != null ? (ref1 = notification.message) != null ? ref1.EventId : void 0 : void 0,
              parentEventId: (notification != null ? (ref2 = notification.message) != null ? ref2.ParentEventId : void 0 : void 0) || null,
              executionId: id,
              date: new Date().toISOString(),
              resource: "" + resource,
              notification: notification,
              user: "" + user,
              clientId: _this3.clientId,
              job: _this3.job,
              app: _this3.app,
              error: _.omit(err, ["detail.request", "cause.detail.request"]),
              request: _.omit(theRequest, _.castArray(_this3.propertiesToOmit).concat("auth")),
              type: _.get(err, "type", "unknown_error"),
              tags: _.get(err, "tags", [])
            })
          };
        });
      }
    }]);

    return MonitoringCenterObserver;
  }();
}).call(undefined);